@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: edit.h,v 1.32 2006/04/20 10:16:04 lav Exp $ */
+/* $Id: edit.h,v 1.34 2008/09/08 11:50:08 lav Exp $ */
 
 #ifndef EDIT_H
 #define EDIT_H
@@ -119,7 +119,8 @@ extern  byte    chset[];
 
 extern int DosEol;
 extern int EolSize;
-extern const char *EolStr;
+extern char EolStr[2];
+void SetEolStr(const char *);
 
 extern   int   TabsInMargin;
 
@@ -144,6 +145,17 @@ extern   int   TabsInMargin;
 #define hex     (editmode==HEXM)
 #define Text    (editmode==TEXT && !buffer_mmapped)
 
+num   GetCol();
+bool  EolAt(offs o);
+bool  BolAt(offs o);
+bool  Eol();
+bool  Bol();
+
+void  DeleteChar();
+void  BackSpace();
+int   InsertChar(char ch);
+int   ReplaceChar(char ch);
+
 void  MoveLeftOverEOL();
 void  MoveRightOverEOL();
 
@@ -311,6 +323,7 @@ int   Suffix(const char *,const char *);
 #define  REDISPLAY_RANGE   8
 
 #include "inline.h"
+#include "mb.h"
 #include "chset.h"
 
 int   isslash(char);
