@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: kern.cc,v 1.31 2008/01/10 17:09:54 lav Exp $ */
+/* $Id: kern.cc,v 1.33 2008/09/08 11:50:08 lav Exp $ */
 
 #include <config.h>
 #include   <stdio.h>
@@ -57,7 +57,7 @@ bool      newfile=false;
 
 int       DosEol=0;
 int       EolSize=1;
-const char *EolStr="\n";
+char	  EolStr[2]="\n";
 
 InodeInfo   FileInfo;
 InodeHistory PositionHistory;
@@ -1067,8 +1067,7 @@ void  EmptyText()
    ScrShift=0;
 
    DosEol=0;
-   EolSize=1;
-   EolStr="\n";
+   SetEolStr("\n");
 }
 
 bool IsAlNumAt(num o)
@@ -1241,3 +1240,52 @@ void  InsertAutoindent(num oldcol)
    }
    stdcol=GetCol();
 }
+
+num GetCol()
+{
+   if(hex)
+      return 0;
+   return(CurrentPos.Col());
+}
+bool EolAt(offs o)
+{
+   return((CharAt(o)==EolStr[0] && (EolSize<2 || CharAt(o+1)==EolStr[1]))
+      || EofAt(o));
+}
+bool BolAt(offs o)
+{
+   return((CharAt(o-EolSize)==EolStr[0] && (EolSize<2 || CharAt(o-1)==EolStr[1]))
+      || BofAt(o));
+}
+bool Eol()
+{
+   return(EolAt(CurrentPos));
+}
+bool Bol()
+{
+   return(BolAt(CurrentPos));
+}
+void DeleteChar()
+{
+   MBCheckRight();
+   DeleteBlock(0,MBCharSize);
+}
+void BackSpace()
+{
+   MBCheckLeft();
+   DeleteBlock(MBCharSize,0);
+}
+int InsertChar(char ch)
+{
+   return(InsertBlock(&ch,1));
+}
+int ReplaceChar(char ch)
+{
+   return ReplaceBlock(&ch,1);
+}
+
+void SetEolStr(const char *n)
+{
+   EolSize=strlen(n);
+   memcpy(EolStr,n,EolSize);
+}
