@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: loadsave.cc,v 1.22 2006/04/19 10:04:51 lav Exp $ */
+/* $Id: loadsave.cc,v 1.25 2008/09/20 05:51:57 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -46,7 +46,7 @@
 #include "edit.h"
 #include "keymap.h"
 #include "highli.h"
-#include <xalloca.h>
+#include <alloca.h>
 #include "block.h"
 #include "clipbrd.h"
 #include "getch.h"
@@ -308,11 +308,10 @@ int   LoadFile(char *name)
       if(UnixLastLine/2<=DosLastLine)
 #endif
       {
-	DosEol=1;
-	EolSize=2;
-	EolStr="\r\n";
-	TextPoint::OrFlags(COLUNDEFINED|LINEUNDEFINED);
-	TextEnd=TextPoint(Size(),DosLastLine,-1);
+	 DosEol=1;
+	 SetEolStr("\r\n");
+	 TextPoint::OrFlags(COLUNDEFINED|LINEUNDEFINED);
+	 TextEnd=TextPoint(Size(),DosLastLine,-1);
       }
       else
       {
