@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: options.cc,v 1.36 2008/01/10 17:09:54 lav Exp $ */
+/* $Id: options.cc,v 1.37 2008/09/02 05:04:45 lav Exp $ */
 
 #include <config.h>
 #include <ctype.h>
@@ -363,11 +363,11 @@ void  ReadConfFromOpenFile(FILE *f,const struct init *init,bool mine)
 	 continue;
       }
       for(i=0; str[i]; i++)
-	 if(!isspace(str[i]))
+	 if(!isspace((unsigned char)str[i]))
 	    break;
       memmove(str,str+i,strlen(str+i)+1);
       for(i=strlen(str); i>0; i--)
-	 if(!isspace(str[i-1]))
+	 if(!isspace((unsigned char)str[i-1]))
 	    break;
       str[i]=0;
       for(i=strlen(str); i>0; i--)
