@@ -16,14 +16,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: screen.cc,v 1.47 2006/11/29 07:36:35 lav Exp $ */
+/* $Id: screen.cc,v 1.48 2008/09/02 05:04:47 lav Exp $ */
 
 #include <config.h>
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
 #include <stdlib.h>
-#include <xalloca.h>
+#include <alloca.h>
 
 #include "edit.h"
 #include "block.h"
